@@ -13,7 +13,7 @@ proc main {} {
     exit 0
 }
 set packages {
-    {app-critcl       ../critcl/critcl.tcl critcl-app}
+    {app-critcl       {.. critcl critcl.tcl} critcl-app}
     {critcl           critcl.tcl}
     {critcl-bitmap    bitmap.tcl}
     {critcl-class     class.tcl}
@@ -68,12 +68,17 @@ proc +x {path} {
 }
 proc critapp {dst} {
     global tcl_platform
-    set app $dst/critcl
+    set app [file join $dst critcl]
     if {$tcl_platform(platform) eq "windows"} {
 	append app .tcl
     }
     return $app
 }
+proc vfile {dir vfile} {
+    global me
+    set selfdir [file dirname $me]
+    eval [linsert $vfile 0 file join $selfdir lib $dir]
+}
 proc grep {file pattern} {
     set lines [split [read [set chan [open $file r]]] \n]
     close $chan
@@ -108,7 +113,7 @@ proc findlib {path} {
 }
 proc dstlfromlib {path} {
     # kinda the inverse of findlib above, it returns the path to
-    # dstlpath, relative the */lib path returned by findlib. The path
+    # dstl relative the */lib path returned by findlib. The path
     # is returned as a list of segments
     set relpath {}
     while {1} {
@@ -116,7 +121,7 @@ proc dstlfromlib {path} {
             break
         }
         set new [file dirname $path]
-        set relpath [linsert $relpath 0 [file tail $path]]
+        set relpath [linsert $relpath[set relpath {}] 0 [file tail $path]]
         if {$new eq $path} break
         set path $new
     }
@@ -134,12 +139,18 @@ proc id {cv vv} {
 }
 proc savedoc {tmpdir} {
     puts {Collecting the documentation ...}
-    file copy -force embedded/www $tmpdir/doc
+    file copy -force [file join embedded www] [file join $tmpdir doc]
     return
 }
+
+proc pkgdirname {name version} {
+	return $name$version
+}
+
+
 proc placedoc {tmpdir} {
     file delete -force doc
-    file copy -force $tmpdir/doc doc
+    file copy -force [file join $tmpdir doc] doc
     return
 }
 proc 2website {} {
@@ -153,6 +164,58 @@ proc reminder {commit} {
     puts ""
     return
 }
+proc shquote value {
+    return "\"[string map [list \\ \\\\ $ \\$ ` \\`] $value]\""
+}
+
+proc targets libdir {
+    if {$libdir eq {} } {
+	set exe  [file dirname [file normalize [file join [info nameofexecutable] ...]]]
+	set dstl [info library]
+	set dsta [file dirname $exe]
+	set dsti [file join [file dirname $dsta] include]
+    } else {
+	set dstl $libdir
+	set libdir [findlib $dstl]
+	set top [file dirname $libdir]
+	set dsta [file join $top bin]
+	set dsti [file join $top include]
+    }
+    list $dsta $dsti $dstl
+}
+
+proc Hsynopsis {} { return "\n\tGenerate a synopsis of procs and builtin types" }
+proc _synopsis {} {
+    puts Public:
+    puts [exec grep -n ^proc lib/critcl/critcl.tcl \
+	      | sed -e "s| \{$||" -e {s/:proc ::critcl::/ /} \
+	      | grep -v { [A-Z]} \
+	      | grep -v { at::[A-Z]} \
+	      | sort -k 2 \
+	      | sed -e {s/^/    /}]
+
+    puts Private:
+    puts [exec grep -n ^proc lib/critcl/critcl.tcl \
+	      | sed -e "s| \{$||" -e {s/:proc ::critcl::/ /} \
+	      | grep {[A-Z]} \
+	      | sort -k 2 \
+	      | sed -e {s/^/    /}]
+
+    puts "Builtin argument types:"
+    puts [exec grep -n {    argtype} lib/critcl/critcl.tcl \
+	      | sed -e "s| \{$||" -e {s/:[ 	]*argtype/ /} \
+	      | sort -k 2 \
+	      | sed -e {s/^/    /}]
+
+    puts "Builtin result types:"
+    puts [exec grep -n {    resulttype} lib/critcl/critcl.tcl \
+	      | sed -e "s| \{$||" -e {s/:[ 	]*resulttype/ /} \
+	      | sort -k 2 \
+	      | sed -e {s/^/    /}]
+
+    return
+}
+
 proc Hhelp {} { return "\n\tPrint this help" }
 proc _help {} {
     usage 0
@@ -171,9 +234,9 @@ proc Htest {} { return "\n\tRun the testsuite." }
 proc _test {} {
     global argv
     set    argv {} ;# clear -- tcltest shall see nothing
-    # Run all .test files in the test/ directory.
+    # Run all .test files in the test directory.
     set selfdir [file dirname $::me]
-    foreach testsuite [lsort -dict [glob -directory $selfdir/test *.test]] {
+    foreach testsuite [lsort -dict [glob -directory [file join $selfdir test] *.test]] {
 	puts ""
 	puts "_ _ __ ___ _____ ________ _____________ _____________________ *** [file tail $testsuite] ***"
 	if {[catch {
@@ -190,23 +253,23 @@ proc _test {} {
 }
 proc Hdoc {} { return "\n\t(Re)Generate the embedded documentation." }
 proc _doc {} {
-    cd [file dirname $::me]/doc
+    cd [file join [file dirname $::me] doc]
 
     puts "Removing old documentation..."
-    file delete -force ../embedded/man
-    file delete -force ../embedded/www
+    file delete -force [file join .. embedded man]
+    file delete -force [file join .. embedded www]
 
-    file mkdir ../embedded/man
-    file mkdir ../embedded/www
+    file mkdir [file join .. embedded man]
+    file mkdir [file join .. embedded www]
 
     puts "Generating man pages..."
-    exec 2>@ stderr >@ stdout dtplite -ext n -o ../embedded/man nroff .
+    exec 2>@ stderr >@ stdout dtplite -ext n -o [file join .. embedded man] nroff .
     puts "Generating html..."
-    exec 2>@ stderr >@ stdout dtplite        -o ../embedded/www html .
+    exec 2>@ stderr >@ stdout dtplite        -o [file join .. embedded www] html .
 
-    cd  ../embedded/man
+    cd  [file join .. embedded man]
     file delete -force .idxdoc .tocdoc
-    cd  ../www
+    cd  [file join .. www]
     file delete -force .idxdoc .tocdoc
 
     return
@@ -215,7 +278,7 @@ proc Htextdoc {} { return "destination\n\tGenerate plain text documentation in s
 proc _textdoc {dst} {
     set destination [file normalize $dst]
 
-    cd [file dirname $::me]/doc
+    cd [file join [file dirname $::me] doc]
 
     puts "Removing old text documentation at ${dst}..."
     file delete -force $destination
@@ -232,7 +295,7 @@ proc _textdoc {dst} {
 }
 proc Hfigures {} { return "\n\t(Re)Generate the figures and diagrams for the documentation." }
 proc _figures {} {
-    cd [file dirname $::me]/doc/figures
+    cd [file join [file dirname $::me] doc figures]
 
     puts "Generating (tklib) diagrams..."
     eval [linsert [glob *.dia] 0 exec 2>@ stderr >@ stdout dia convert -t -o . png]
@@ -261,27 +324,27 @@ proc _release {} {
 
     # # ## ### ##### ######## #############
     puts {Generate starkit...}
-    _starkit $tmpdir/critcl31.kit
+    _starkit [file join $tmpdir critcl31.kit]
 
     # # ## ### ##### ######## #############
     puts {Collecting starpack prefix...}
     # which we use the existing starpack for, from the gh-pages branch
 
     exec 2>@ stderr >@ stdout git checkout gh-pages
-    file copy download/critcl31.exe $tmpdir/prefix.exe
+    file copy [file join download critcl31.exe] [file join $tmpdir prefix.exe]
     exec 2>@ stderr >@ stdout git checkout $commit
 
     # # ## ### ##### ######## #############
     puts {Generate starpack...}
-    _starpack $tmpdir/prefix.exe $tmpdir/critcl31.exe
+    _starpack [file join $tmpdir prefix.exe] [file join $tmpdir critcl31.exe]
     # TODO: vacuum the thing. fix permissions if so.
 
     # # ## ### ##### ######## #############
     2website
     placedoc $tmpdir
 
-    file copy -force $tmpdir/critcl31.kit download/critcl31.kit
-    file copy -force $tmpdir/critcl31.exe download/critcl31.exe
+    file copy -force [file join $tmpdir critcl31.kit] [file join downloadcritcl31.kit]
+    file copy -force [file join $tmpdir critcl31.exe] [file join download critcl31.exe]
 
     set index   [fileutil::cat index.html]
     set pattern   "\\\[commit .*\\\] \\(v\[^)\]*\\)<!-- current"
@@ -314,13 +377,20 @@ proc _release-doc {} {
 }
 
 proc Htargets {} { return "?destination?\n\tShow available targets.\n\tExpects critcl app to be installed in destination." }
-proc _targets {{dst {}}} {
-    if {[llength [info level 0]] < 2} {
-	set dsta [file dirname [file dirname [file normalize [info nameofexecutable]/___]]]
-    } else {
-	set dsta [file dirname [findlib $dstl]]/bin
+proc _targets args {
+    switch [llength $args] {
+	0 - 1 {
+	}
+	default {
+	    error -list wrong # args
+	}
     }
-    puts [join [split [exec $dsta/critcl -targets]] \n]
+    if {[llength [info level 0]] < 2} {
+	lassign [targets {}] dsta dsti dstl
+    } else {
+	lassign [targets [file join [file dirname [lindex [info level 0] 1]] lib]] dsta dsti dstl
+    }
+    puts [join [split [exec [file join $dsta critcl] -targets]] \n]
     return
 }
 
@@ -335,18 +405,20 @@ proc _install {args} {
     }
 
     if {[llength $args] == 0} {
-	set dstl [info library]
-	set dsta [file dirname [file dirname [file normalize [info nameofexecutable]/___]]]
+	set libdir {}
+
     } else {
-	set dstl [lindex $args 0]
-	set dsta [file dirname [findlib $dstl]]/bin
+	set libdir [lindex $args 0]
     }
+    lassign [targets $libdir] dsta dsti dstl
+    file mkdir $dsta $dsti
 
     set selfdir [file dirname $me]
 
     puts {Installing into:}
     puts \tPackages:\t$dstl
     puts \tApplication:\t$dsta
+    puts \tHeaders:\t$dsti
 
     if {[catch {
 	# Create directories, might not exist.
@@ -368,15 +440,17 @@ proc _install {args} {
 	    }
 
 	    if {$vfile ne {}} {
-		set version  [version $selfdir/lib/$dir/$vfile]
+		set version [version [vfile $dir $vfile]]
 	    } else {
 		set version {}
 	    }
 
-	    file copy   -force $selfdir/lib/$dir     $dstl/${name}-new
-	    file delete -force $dstl/$name$version
-	    file rename        $dstl/${name}-new     $dstl/$name$version
-	    puts "${prefix}Installed package:      $dstl/$name$version"
+	    set namevers [file join $dstl [pkgdirname $name $version]]
+
+	    file copy -force [file join $selfdir lib $dir] [file join $dstl ${name}-new]
+	    file delete -force $namevers
+	    puts "${prefix}Installed package:      $namevers"
+	    file rename [file join $dstl ${name}-new] $namevers
 	    set prefix {}
 	}
 
@@ -387,17 +461,23 @@ proc _install {args} {
 
 	set c [open $theapp w]
 	lappend map @bs@   "\\"
-	lappend map @exe@  [file dirname [file normalize [info nameofexecutable]/___]]
-	lappend map @path@ $reldstl  ;# insert the dst path
+	lappend map @exe@ [shquote [file dirname [file normalize [
+	    file join [info nameofexecutable] ...]]]]
+	lappend map @path@ [list $reldstl]  ;# insert the dst path
+	puts [list geedonk $reldstl]
 	lappend map "\t    " {} ;# de-dent
 	puts $c [string trimleft [string map $map {
 	    #!/bin/sh
-	    # -*- tcl -*- @bs@
-	    exec "@exe@" "$0" ${1+"$@"}
+	    # -*-tcl -*-
+	    # hide next line from tcl @bs@
+	    exec @exe@ "$0" ${1+"$@"}
 
-	    set libpath [file normalize [file join [file dirname [info script]] .. lib]]
-	    set libpath [file join $libpath {@path@}]
-	    if {[lsearch -exact $auto_path $libpath] < 0} {lappend auto_path $libpath}
+	    set libpath [file join [file dirname [file dirname [
+		file normalize [file join [info script] ...]]]] .. lib]
+	    set libpath [file join $libpath @path@]
+	    if {[lsearch -exact $auto_path $libpath] < 0} {
+		set auto_path [linsert $auto_path[set auto_path {}] 0 $libpath]
+	    }
 
 	    package require critcl::app
 	    critcl::app::main $argv}]]
@@ -409,6 +489,8 @@ proc _install {args} {
 	# Special package: critcl_md5c
 	# Local MD5 hash implementation.
 
+	puts "\nInstalled C package:\tcritcl::md5c"
+
 	# It is special because it is a critcl-based package, not pure
 	# Tcl as everything else of critcl. Its installation makes it
 	# the first package which will be compiled with critcl on this
@@ -416,28 +498,67 @@ proc _install {args} {
 	# that. This is package-mode, where MD5 itself is not used, so
 	# there is no chicken vs. egg.
 
-	set src     $selfdir/lib/critcl-md5c/md5c.tcl
+	set src     [file join $selfdir lib critcl-md5c md5c.tcl]
 	set version [version $src]
 	set name    critcl_md5c
-	set dst     $dstl/$name$version
+	set dst     [file join $dstl [pkgdirname $name $version]]
+	set cmd     {}
 
 	lappend cmd exec >@ stdout 2>@ stderr
-	if {$::tcl_platform(platform) eq "windows"} {
-	    lappend cmd [info nameofexecutable]
-	}
+	lappend cmd [info nameofexecutable]
 	lappend cmd $theapp
 	if {$target ne {}} {
 	    lappend cmd -target $target
 	}
-	lappend cmd -libdir $dstl/tmp -pkg $src
+	lappend cmd -libdir [file join $dstl tmp] -pkg $src
+	puts [list executing $cmd]
 	eval $cmd
 
 	file delete -force $dst
-	file rename        $dstl/tmp/md5c $dst
-	file delete -force $dstl/tmp
+	file rename        [file join $dstl tmp md5c] $dst
+	file delete -force [file join $dstl tmp]
 
 	puts "${prefix}Installed package:      $dst"
 
+	# Special package: critcl::callback
+	# C/Tcl callback utility code.
+
+	puts "\nInstalled C package:\tcritcl::callback"
+
+	# It is special because it is a critcl-based package, not pure
+	# Tcl as everything else of critcl. Its installation makes it
+	# the second package which will be compiled with critcl on this
+	# machine. It uses the just-installed application for
+	# that.
+
+	set src     [file join $selfdir lib critcl-callback callback.tcl]
+	set version [version $src]
+	set name    critcl_callback
+	set dst     [file join $dstl $name$version]
+	set dsth    [file join $dsti $name]
+	set cmd     {}
+
+	lappend cmd exec >@ stdout 2>@ stderr
+	lappend cmd [info nameofexecutable]
+	lappend cmd $theapp
+	if {$target ne {}} {
+	    lappend cmd -target $target
+	}
+	set dstl_tmp [file join $dstl tmp]
+	lappend cmd -libdir     $dstl_tmp
+	lappend cmd -includedir $dstl_tmp
+	lappend cmd -pkg $src
+	eval $cmd
+
+	file delete -force $dst $dsth
+	file rename  [file join $dstl tmp callback] $dst
+	file rename  [file join $dstl tmp critcl_callback] $dsth
+	file delete -force $dstl_tmp
+
+	puts "${prefix}Installed package:      $dst"
+	puts "${prefix}Installed headers:      [
+	    file join $dsti critcl_callback]"
+
     } msg]} {
 	if {![string match {*permission denied*} $msg]} {
 	    return -code error -errorcode $::errorCode -errorinfo $::errorInfo $msg
@@ -448,24 +569,27 @@ proc _install {args} {
     return
 }
 proc Huninstall {} { Hdrop }
-proc _uninstall {args} { _drop {*}$args }
+proc _uninstall {args} { eval [linsert $args 0 _drop] }
 
 proc Hdrop {} { return "?destination?\n\tRemove packages.\n\tdestination = path of package directory, default \[info library\]." }
 proc _drop {{dst {}}} {
-    global packages
+    global packages me
 
     if {[llength [info level 0]] < 2} {
 	set dstl [info library]
-	set dsta [file dirname [file dirname [file normalize [info nameofexecutable]/___]]]
+	set dsta [file dirname [file dirname [file normalize [file join [
+	    info nameofexecutable] ...]]]]
     } else {
 	set dstl $dst
-	set dsta [file dirname $dst]/bin
+	set dsta [file join [file dirname $dst] bin]
     }
 
     # Add the special package (see install). Not special with regard
     # to removal. Except for the name
     lappend packages [list critcl-md5c md5c.tcl critcl_md5c]
 
+    set selfdir [file dirname $me]
+
     foreach item $packages {
 	# Package: /name/
 
@@ -481,13 +605,15 @@ proc _drop {{dst {}}} {
 	}
 
 	if {$vfile ne {}} {
-	    set version  [version [file dirname $::me]/lib/$dir/$vfile]
+	    set version [version [vfile $dir $vfile]]
 	} else {
 	    set version {}
 	}
 
-	file delete -force $dstl/$name$version
-	puts "Removed package:     $dstl/$name$version"
+	set namevers [file join $dstl $name$version]
+
+	file delete -force $namevers
+	puts "Removed package:     $namevers"
     }
 
     # Application: critcl
@@ -521,11 +647,11 @@ proc _starpack {prefix {dst critcl}} {
     file copy -force $prefix $dst
 
     vfs::mk4::Mount $dst /KIT
-    file mkdir /KIT/lib
+    file mkdir [file join /KIT lib]
 
     foreach d [glob -directory lib *] {
-	file delete -force  /KIT/lib/[file tail $d]
-	file copy -force $d /KIT/lib
+	file delete -force  [file join /KIT lib [file tail $d]]
+	file copy -force $d [file join /KIT lib]
     }
 
     file copy -force main.tcl /KIT
@@ -543,7 +669,7 @@ proc _examples {args} {
 
     # List examples, or run the build code on the examples, passing any arguments.
 
-    set examples [glob -directory $selfdir/examples */$self]
+    set examples [lsort -dict [glob -directory [file join $selfdir examples] */$self]]
 
     puts ""
     if {![llength $args]} {
